SystemOrganization addCategory: #'Amop-Proxy'!
SystemOrganization addCategory: #'Amop-Core'!
SystemOrganization addCategory: #'Amop-Core-Tests'!
SystemOrganization addCategory: #'Amop-Compiler'!

RBProgramNodeVisitor subclass: #AmopVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Compiler'!

!AmopVisitor methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 10:53'!
acceptMessageNode: messageNode 

	( messageNode isNew ) ifTrue: [
		^ self acceptNewMessageNode: messageNode  
	].

	( messageNode isSelfSend ) ifTrue: [  
		 "if logging, special transformation for self and super sends"
		^self acceptSelfAndSuperMessageNode: messageNode.
	].
	
	^self acceptNormalMessageNode: messageNode.! !

!AmopVisitor methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 12:06'!
acceptNewMessageNode: messageNode 

	(messageNode needsAmopRewrite) ifTrue: [
		| myself selector sender arguments proxyMessageNode chainMessageNode orignalMessageNode |
			
		myself := self buildVariableNode: 'self'. 
		selector := self buildLiteralNode: messageNode selector. 
		arguments := RBArrayNode statements: messageNode arguments.
				
		proxyMessageNode := RBMessageNode receiver:  myself
		 	selector: #receiverProxy
			arguments: { }. 
			
		orignalMessageNode := RBMessageNode receiver: proxyMessageNode
			selector: #newTrap:
			arguments: { 
				RBMessageNode receiver:  messageNode receiver
		 		selector: #new
				arguments: { } .
			 }.
		
		messageNode replaceWith: orignalMessageNode.		
		^ super visitNode: orignalMessageNode.
	].

	^ super acceptMessageNode: messageNode.! !

!AmopVisitor methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 10:40'!
acceptNormalMessageNode: messageNode 

	^ super acceptMessageNode: messageNode.! !

!AmopVisitor methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 11:47'!
acceptSelfAndSuperMessageNode: messageNode 

	(messageNode needsAmopRewrite) ifTrue: [
		| myself selector sender arguments proxyMessageNode orignalMessageNode |
			
		myself := self buildVariableNode: 'self'. 
		selector := self buildLiteralNode: messageNode selector. 
		arguments := RBArrayNode statements: messageNode arguments.
				
		proxyMessageNode := RBMessageNode receiver:  myself
		 	selector: #receiverProxy 
			arguments: {}.
		
		orignalMessageNode := RBMessageNode receiver:  proxyMessageNode
		 	selector: messageNode selector    
			arguments: messageNode arguments.
		
		messageNode replaceWith: orignalMessageNode.		
		^ super visitNode: orignalMessageNode.
	].

	^ super acceptMessageNode: messageNode.! !

!AmopVisitor methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 11:43'!
buildLiteralNode: aName
	| literal |
	literal  := RBLiteralToken value: aName.
	^ RBLiteralValueNode literalToken: literal.! !

!AmopVisitor methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 11:44'!
buildVariableNode: aName
	| literal |
	literal  := RBLiteralToken value: aName.
	^ RBVariableNode identifierToken: literal.! !

RBProgramNodeVisitor subclass: #OwnershipCompilerMessageVisitor3
	instanceVariableNames: 'skip'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Compiler'!

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'ewe 6/22/2012 14:00'!
acceptArrayNode: arrayNode 
	| newArrayNode newMessageNode |

	"Exclude certain nodes (see acceptMethodNode)"
	skip ifTrue: [ ^ super acceptArrayNode: arrayNode. ].
	
	(arrayNode isOwnershipArrayNode) ifFalse: [
	
		newArrayNode:= RBOwnershipArrayNode statements: arrayNode statements.
	
		newMessageNode:= RBMessageNode receiver: (self buildVariableNode: 'DOArray')
			selector: #withAll:
			arguments: (OrderedCollection withAll: newArrayNode).
	
		arrayNode replaceWith: newMessageNode.	
		
		^ super visitNode: newMessageNode.
	
	].
	
	^ super acceptArrayNode: arrayNode.! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'ewe 6/22/2012 13:59'!
acceptLiteralArrayNode: literalArrayNode 
	| newArrayNode newMessageNode |

	"Exclude certain nodes (see acceptMethodNode)"
	skip ifTrue: [ ^ super acceptLiteralArrayNode: literalArrayNode. ].
	
	(literalArrayNode isOwnershipLiteralArrayNode) ifFalse: [

		newArrayNode:= RBOwnershipArrayNode statements: literalArrayNode contents.

		newMessageNode:= RBMessageNode receiver: (self buildVariableNode: 'DOArray')
			selector: #withAll:
			arguments: (OrderedCollection withAll: newArrayNode).

		literalArrayNode replaceWith: newMessageNode.
		
		^ super visitNode: newMessageNode.
	
	].
	^ super acceptLiteralArrayNode: literalArrayNode.! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'ewe 6/27/2012 14:47'!
acceptMessageNode: messageNode 
	| logging |
	"logging = true: self and super will be converted to  self track: [ original method ] selector: #selector"
	logging:= false.
	
	"Excluse certain methods (see acceptMethodNode)"
	skip ifTrue: [ ^super acceptMessageNode: messageNode. ].

	( logging and: [ messageNode isSelfSend or: [messageNode isSuperSend ] ] ) ifTrue: [  
		 "if logging, special transformation for self and super sends"
		^self acceptSelfAndSuperMessageNode: messageNode.
	]
	ifFalse: [
		"normal case"
		^self acceptNormalMessageNode: messageNode.
	].! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'pmaer 7/10/2012 17:05'!
acceptMethodNode: aMethod
	VariableNameCounter resetCounter. 
	skip := aMethod isOwernshipMeta.
	^ super acceptMethodNode: aMethod.
	! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'ewe 6/28/2012 09:14'!
acceptNormalMessageNode: messageNode 

	"Exclude self and super sends and few other cases"
	(messageNode isExcluded ) 
		ifTrue: [ ^super acceptMessageNode: messageNode. ].	
		
				
	(messageNode needsOwnershipRewrite) ifTrue: [
		| block selector sender arguments newMessageNode |
			
		"block := self buildVariableNode: 'self'. "
		block := self buildSyntheticBlock: messageNode .
		selector := self buildLiteralNode: messageNode selector. 
		sender := self buildVariableNode: 'self'.
		arguments := RBOwnershipArrayNode statements: messageNode arguments.
				
		newMessageNode:= RBMessageNode receiver:  messageNode receiver
		 	selector: #checkValidSend3:message:block:arguments:
			arguments: { sender. selector. block. arguments}.
		
		messageNode replaceWith: newMessageNode.		
		^ super visitNode: newMessageNode.
	].

	^ super acceptMessageNode: messageNode.! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'pmaer 6/26/2012 16:10'!
acceptSelfAndSuperMessageNode: messageNode 

"Send type logger, special block for this case:"
	
	"Exclude self thisSender"
	(messageNode isSelfSend and: [messageNode selector = #thisSender]) ifTrue: [
		 ^super acceptMessageNode: messageNode.
		].
	
	 ((messageNode isSelfSend or: [ messageNode isSuperSend ])  and: [messageNode needsOwnershipRewrite ])
		ifTrue: [ 
			| block selector arguments newMessageNode receiver |
			
			block:= self buildTrackBlock: messageNode .
			selector := self buildLiteralNode: messageNode selector. 
				
			newMessageNode:= RBMessageNode receiver:  (self buildVariableNode: 'self')
		 	selector: #track:selector:
			arguments: { block. selector.}.
		
			messageNode replaceWith: newMessageNode.
			
			^ super visitNode: newMessageNode.
			
			
			].
	"------------------------------------------"
	^ super acceptMessageNode: messageNode.! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'ewe 4/25/2012 14:58'!
buildLiteralNode: aName
	| literal |
	literal  := RBLiteralToken value: aName.
	^ RBLiteralValueNode literalToken: literal.! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'ewe 4/25/2012 15:33'!
buildSyntheticBlock: messageNode 
| blockBodyMessageArguments blockArguments block blockBody collStatements blockBodyMessage  blockBodyReceiver blockBodyReceiverName blockFirstArg |

		"Build a block of the form
		  [ :receiver :arg | receiver selector: arg ]
		"
		
		" Build the message that will be in the synthetic block"
		blockBodyReceiverName := self nextVariableName.
		blockBodyReceiver := self buildVariableNode: blockBodyReceiverName. 
		blockBodyMessageArguments:= OrderedCollection new.
		blockArguments := OrderedCollection new.
		messageNode arguments size timesRepeat: [ 
			| argName |
			argName := self nextVariableName .
			blockBodyMessageArguments addLast: ( self buildVariableNode: argName ).
			blockArguments addLast: ( self buildVariableNode: argName ).
		].
		blockBodyMessage := RBMessageNode receiver: blockBodyReceiver
			selector: messageNode selector
			arguments: blockBodyMessageArguments.
		
		"Now build the synthetic block itself"
		collStatements := OrderedCollection new.
		collStatements add: blockBodyMessage.
		blockBody := RBSequenceNode statements: collStatements.
		
		blockFirstArg := self buildVariableNode: blockBodyReceiverName. 
		blockArguments addFirst: blockFirstArg.
		
		block := RBOwnershipBlockNode 
			arguments: blockArguments
			body: blockBody. 
			
		^ block
		! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'pmaer 6/26/2012 12:09'!
buildTrackBlock: messageNode 
| blockBodyMessageArguments blockArguments block blockBody collStatements blockBodyMessage  blockBodyReceiver blockBodyReceiverName blockFirstArg |

		"Build a block of the form
		  [ :receiver :arg | receiver selector: arg ]
		"
		
		" Build the message that will be in the synthetic block"
		blockBodyReceiver := messageNode receiver.
		blockBodyMessageArguments:= messageNode arguments.

		blockBodyMessage := RBMessageNode receiver: blockBodyReceiver
			selector: messageNode selector
			arguments: blockBodyMessageArguments.
		
		"Now build the synthetic block itself"
		collStatements := OrderedCollection new.
		collStatements add: blockBodyMessage.
		blockBody := RBSequenceNode statements: collStatements.
		
		block := RBOwnershipBlockNode
			body: blockBody. 
		^ block
		! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'ewe 4/25/2012 14:30'!
buildVariableNode: aName
	| literal |
	literal  := RBLiteralToken value: aName.
	^ RBVariableNode identifierToken: literal.! !

!OwnershipCompilerMessageVisitor3 methodsFor: 'as yet unclassified' stamp: 'ewe 4/25/2012 14:29'!
nextVariableName
	^ VariableNameCounter nextVariableName 
	! !

!RBMessageNode methodsFor: '*amop' stamp: 'ewe 8/30/2012 10:51'!
isNew
	^ self selector = #new! !

TestCase subclass: #TestTransparency
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Core-Tests'!

!TestTransparency methodsFor: 'as yet unclassified' stamp: 'ewe 8/31/2012 11:26'!
setUp
	ProxyHandler receiverProxy: nil handler: ProxyHandler new. ! !

!TestTransparency methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 14:10'!
testReturn42
	self assert: ( TestObject newp return42 = 42 ).
	self assert: ( TestObject newp selfReturn42 = 42 ).
	self assert: ( SubTestObject newp return42 = 42 ).
	self assert: ( SubTestObject newp selfReturn42 = 42 ).  "No correctly hanlding super"! !

!TestTransparency methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 14:10'!
testReturnObject
	self assert: ( TestObject newp returnObject = nil ) not .
	self assert: ( TestObject newp selfReturnObject = nil ) not .
	self assert: ( SubTestObject newp returnObject = nil ) not .
	self assert: ( SubTestObject newp selfReturnObject = nil ) not . ! !

RBCompiler subclass: #AmopCompiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Compiler'!

!AmopCompiler methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 10:02'!
translate: aProgramNode
AmopVisitor new visitNode: aProgramNode methodNode..
^ super translate: aProgramNode.
! !

RBCompiler subclass: #OwnershipCompiler3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Compiler'!

!OwnershipCompiler3 methodsFor: 'as yet unclassified' stamp: 'pmaer 3/26/2012 19:20'!
translate: aProgramNode
OwnershipCompilerMessageVisitor3 new visitNode: aProgramNode methodNode..
^ super translate: aProgramNode.
! !

Object subclass: #BaseObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Core'!

!BaseObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 14:16'!
receiverProxy 
	"TODO walk up the chain until the LAST proxy, consider chains of proxy and super sends"
	"self receiverProxy returns the caller of the current activation"
	^ ProxyHandler receiverProxy.! !

!BaseObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 11:47'!
test
	^ self xxx     ! !

BaseObject subclass: #TestObject
	instanceVariableNames: 'helper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Core-Tests'!

TestObject subclass: #SubTestObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Core-Tests'!

!SubTestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 12:02'!
return42
	^super return42.! !

!SubTestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 10:48'!
returnObject 
	^super returnObject.! !

!SubTestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 10:47'!
selfReturn42
	^ super selfReturn42 ! !

!SubTestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 10:48'!
selfReturnObject
	^ super selfReturnObject! !

!TestObject class methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 12:02'!
compilerClass 
	^AmopCompiler! !

!TestObject class methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 12:16'!
newp
	^ Proxy for: self new. ! !

!TestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 17:17'!
passParameterToHelper: aValue
 	^ helper compute: aValue . ! !

!TestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 17:16'!
passReturnValueFromHelper

 	^ helper return42. ! !

!TestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 12:01'!
return42
	^ 42! !

!TestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 12:04'!
returnObject
	^ Object new. ! !

!TestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/28/2012 12:15'!
selfReturn42
	^ self return42 ! !

!TestObject methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 10:47'!
selfReturnObject
	^ self returnObject  ! !

Object subclass: #OwnedObject3
	uses: TOwnershipCheck
	instanceVariableNames: 'dynamicOwner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Core'!

OwnedObject3 class
	uses: TOwnershipCheck classTrait
	instanceVariableNames: ''!

!OwnedObject3 methodsFor: 'transfer' stamp: 'ewe 5/2/2012 16:45'!
borrow: anObject do: aBlock
	| oldOwner |
	oldOwner := anObject owner. 
	anObject owner: self. 
	[
	  ^aBlock value: anObject. 
	] ensure: [
		anObject owner: oldOwner. 
	]! !

!OwnedObject3 methodsFor: 'testing' stamp: 'ewe 4/20/2012 11:20'!
canNotSend: selector to: receiver
	[ self canSend: selector to: receiver ] on: MessageFiltered do: [ ^ true ]. 
	^ false. 
	
! !

!OwnedObject3 methodsFor: 'testing' stamp: 'ewe 4/19/2012 16:10'!
canSend: selector to: receiver
	"Tests whether the object can send the message to the receiver"
	self checkValidSend3: receiver message: selector block: [:t | ^ true] arguments: {}.
	^ false. 
	
! !

!OwnedObject3 methodsFor: 'transfer' stamp: 'ewe 6/25/2012 15:01'!
copy
	<meta>
	| copy |
	copy := super copy. 
	copy ownerNoCheck: self owner. 
	^ copy. ! !

!OwnedObject3 methodsFor: 'enforcement' stamp: 'ewe 7/5/2012 15:32'!
isAccessibleFrom: aSource
	| interfaces |
	interfaces := self class allPragmas .
	self handleFilters: aSource selector: #any interfaces: interfaces. 
	
! !

!OwnedObject3 methodsFor: 'enforcement' stamp: 'ewe 7/5/2012 15:52'!
isAccessibleFromAllReferencs

	"This check is too slow in practice"
	true ifTrue: [ ^self ].
	
	self pointersTo do: [ :source | 
		( source isKindOf: OwnedObject3 ) ifTrue: [
			"Transcript show: 'Check from' ; show: source class asString ; cr.  "	
			self isAccessibleFrom: source ]
	]! !

!OwnedObject3 methodsFor: 'enforcement' stamp: 'ewe 6/27/2012 17:26'!
owner
	dynamicOwner isNil ifTrue: [ dynamicOwner := WorldOwner getInstance ].
	^ dynamicOwner. ! !

!OwnedObject3 methodsFor: 'transfer' stamp: 'ewe 7/5/2012 15:30'!
owner: newOwner
	"Transcript show:  'Attempt to transfer ownership (object: ' ; show: self class printString ;
	show: ', candidate owner: ' ; show:  newOwner class printString ; show:  ')' ; cr ."
	(newOwner acceptOwnershipOf: self ) ifTrue: [
		WorldOwner resetPathCache.
		dynamicOwner := newOwner. 
		"Transcript show:  'Ownership transfer succeeded' ; cr ."
		self isAccessibleFromAllReferencs.
		
	] ifFalse: [ 
		Transcript show: thisContext traceString  ; cr. 
		TransferError signal
	 ].
	"TODO the method does not return itself, as this might create violations"
	 ^nil. ! !

!OwnedObject3 methodsFor: 'enforcement' stamp: 'pmaer 7/4/2012 15:20'!
ownerNoCheck: newOwner
	"TODO this one should ideally never be used. Momentary necessary because of instrumented / not instrumente d code"
	"setter for owner"
	WorldOwner resetPathCache.
	dynamicOwner := newOwner.! !

Object subclass: #Proxy
	instanceVariableNames: 'handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Proxy'!

!Proxy class methodsFor: 'as yet unclassified' stamp: 'ewe 8/31/2012 11:47'!
for: aTarget
	| proxy |
	proxy := self handler: ( ProxyHandler target: aTarget ) .
	ProxyTrap adoptInstance:  proxy. 
	^ proxy. ! !

!Proxy class methodsFor: 'as yet unclassified' stamp: 'ewe 8/31/2012 11:47'!
handler: aHandler
	^ self new handler: aHandler ; yourself.! !

!Proxy methodsFor: 'as yet unclassified' stamp: 'ewe 8/31/2012 11:49'!
cannotInterpret: aMessage
	| sender receiver senderHandler receiverHandler target |
	
	Transcript show: aMessage selector asString ; cr.
	
	target := handler target.  
	
	"sender of this message is the old receiver"
	sender := ProxyHandler receiverProxy. 
	senderHandler := ProxyHandler receiverProxyHandler.
	"now swap the receiver"
	receiver := self.
	receiverHandler := handler.
	ProxyHandler receiverProxy:  receiver handler: receiverHandler.. 
	
	( aMessage selector = #newTrap: ) ifTrue: [
		^ handler newTrapContinue: [ Proxy for: aMessage arguments first ].
	].
	( aMessage selector = #readField: ) ifTrue: [
		^ handler readField: aMessage arguments first. 
	].
	( aMessage selector = #writeField: ) ifTrue: [
		^ handler writeField: aMessage arguments first. 
	].

	"Invoke 
	(1) param out on the sender, (2) param in on the receiver, 
	(3) delegate processing of the message
	(4) return out on the receiver, (5) return in on the sender "

	^ senderHandler messageOut: aMessage receiver: receiverHandler continue: [  
		receiverHandler messageIn: aMessage sender: senderHandler continue: [ 
			target perform: aMessage selector withArguments: aMessage arguments.  ] ].! !

!Proxy methodsFor: 'as yet unclassified' stamp: 'ewe 8/31/2012 11:45'!
handler: aHandler
	handler := aHandler. ! !

Proxy subclass: #ProxyTrap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Amop-Proxy'!

!ProxyTrap class methodsFor: 'as yet unclassified' stamp: 'ewe 8/24/2012 16:04'!
initialize 
	methodDict := nil. ! !

Object subclass: #ProxyHandler
	instanceVariableNames: 'target'
	classVariableNames: 'Receiver ReceiverHandler'
	poolDictionaries: ''
	category: 'Amop-Proxy'!

!ProxyHandler class methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 14:16'!
receiverProxy
	^Receiver ! !

!ProxyHandler class methodsFor: 'as yet unclassified' stamp: 'ewe 8/30/2012 17:22'!
receiverProxy: anObject 
	^ Receiver := anObject.! !

!ProxyHandler class methodsFor: 'as yet unclassified' stamp: 'ewe 8/31/2012 11:21'!
receiverProxy: anObject handler: aHandler
	 Receiver := anObject.
	  ReceiverHandler:=aHandler.! !

!ProxyHandler class methodsFor: 'as yet unclassified' stamp: 'ewe 8/31/2012 11:20'!
receiverProxyHandler
	^ReceiverHandler! !

!ProxyHandler class methodsFor: 'as yet unclassified' stamp: 'ewe 8/31/2012 11:47'!
target: aTarget
	^ self new target: aTarget ; yourself. ! !

!ProxyHandler methodsFor: 'trap' stamp: 'ewe 8/30/2012 12:09'!
fieldRead: anAccess! !

!ProxyHandler methodsFor: 'trap' stamp: 'ewe 8/30/2012 12:09'!
fieldWrite: anAccess! !

!ProxyHandler methodsFor: 'trap' stamp: 'ewe 8/31/2012 11:43'!
messageIn: aMessage sender: senderHandler continue: proceed
	^ proceed value.! !

!ProxyHandler methodsFor: 'trap' stamp: 'ewe 8/31/2012 11:43'!
messageOut: aMessage receiver: receiverHandler continue: proceed
	^ proceed value. ! !

!ProxyHandler methodsFor: 'trap' stamp: 'ewe 8/31/2012 10:46'!
newTrapContinue: proceed 
	^ proceed value.! !

!ProxyHandler methodsFor: 'accessor' stamp: 'ewe 8/31/2012 11:47'!
target
	^ target.! !

!ProxyHandler methodsFor: 'accessor' stamp: 'ewe 8/31/2012 11:47'!
target: aTarget
	target := aTarget.! !
ProxyTrap initialize!
